!-------------------------------------------------------------------
! manages reading and interpolation of prescribed ozone
! Created by: Francis Vitt
!-------------------------------------------------------------------
module prescribed_ozone

  use shr_kind_mod, only : r8 => shr_kind_r8
  use abortutils,   only : endrun
  use spmd_utils,   only : masterproc
  use tracer_data,  only : trfld, trfile
  use cam_logfile,  only : iulog

  implicit none
  private
  save 

  type(trfld), pointer :: fields(:)
  type(trfile)         :: file

  public :: prescribed_ozone_init
  public :: prescribed_ozone_adv
  public :: write_prescribed_ozone_restart
  public :: read_prescribed_ozone_restart
  public :: has_prescribed_ozone
  public :: prescribed_ozone_defaultopts
  public :: prescribed_ozone_setopts
  public :: prescribed_ozone_register
  public :: init_prescribed_ozone_restart

  logical :: has_prescribed_ozone = .false.
  character(len=8), parameter :: ozone_name = 'ozone'

  character(len=16)  :: fld_name = 'ozone'
  character(len=256) :: filename = ''
  character(len=256) :: filelist = ''
  character(len=256) :: datapath = ''
  character(len=32)  :: data_type = 'SERIAL'
  logical            :: rmv_file = .false.
  integer            :: start_ymd = 0
  integer            :: start_tod = 0

contains

!-------------------------------------------------------------------
!-------------------------------------------------------------------
  subroutine prescribed_ozone_register()
    use ppgrid,         only: pver
    use phys_buffer,    only: pbuf_add
    integer :: oz_idx

    if (has_prescribed_ozone) then
       call pbuf_add(ozone_name,'physpkg',1,pver,1,oz_idx)
    endif

  endsubroutine prescribed_ozone_register

!-------------------------------------------------------------------
!-------------------------------------------------------------------
  subroutine prescribed_ozone_init()

    use tracer_data, only : trcdata_init
    use cam_history, only : addfld, phys_decomp
    use ppgrid,      only : pver
    use error_messages, only: handle_err
    use ppgrid,         only: pcols, pver, begchunk, endchunk

    implicit none

    integer :: ndx, istat
    character(len=32) :: specifier(1)
    
    if ( has_prescribed_ozone ) then
       if ( masterproc ) then
          write(iulog,*) 'ozone is prescribed in :'//trim(filename)
       endif
    else
       return
    endif

    specifier(1) = trim(ozone_name)//':'//trim(fld_name)

    file%in_pbuf = .true.
    call trcdata_init( specifier, filename, filelist, datapath, fields, file, &
                       rmv_file, start_ymd, start_tod, data_type )

    call addfld(ozone_name,'mol/mol ', pver, &
         'I', 'prescribed ozone', phys_decomp )

  end subroutine prescribed_ozone_init

!-------------------------------------------------------------------
!-------------------------------------------------------------------
  subroutine prescribed_ozone_setopts(       &
       prescribed_ozone_file_in,      &
       prescribed_ozone_filelist_in,  &
       prescribed_ozone_datapath_in,  &
       prescribed_ozone_type_in,      &
       prescribed_ozone_name_in, &
       prescribed_ozone_rmfile_in,    &
       prescribed_ozone_ymd_in,       &
       prescribed_ozone_tod_in        &
       )

    implicit none

    character(len=*), intent(in), optional :: prescribed_ozone_file_in
    character(len=*), intent(in), optional :: prescribed_ozone_filelist_in
    character(len=*), intent(in), optional :: prescribed_ozone_datapath_in
    character(len=*), intent(in), optional :: prescribed_ozone_type_in
    character(len=*), intent(in), optional :: prescribed_ozone_name_in
    logical,          intent(in), optional :: prescribed_ozone_rmfile_in
    integer,          intent(in), optional :: prescribed_ozone_ymd_in
    integer,          intent(in), optional :: prescribed_ozone_tod_in

    if ( present(prescribed_ozone_file_in) ) then
       filename = prescribed_ozone_file_in
    endif
    if ( present(prescribed_ozone_filelist_in) ) then
       filelist = prescribed_ozone_filelist_in
    endif
    if ( present(prescribed_ozone_datapath_in) ) then
       datapath = prescribed_ozone_datapath_in
    endif
    if ( present(prescribed_ozone_type_in) ) then
       data_type = prescribed_ozone_type_in
    endif
    if ( present(prescribed_ozone_name_in) ) then
       fld_name = prescribed_ozone_name_in
    endif
    if ( present(prescribed_ozone_rmfile_in) ) then
       rmv_file = prescribed_ozone_rmfile_in
    endif
    if ( present(prescribed_ozone_ymd_in) ) then
       start_ymd = prescribed_ozone_ymd_in
    endif
    if ( present(prescribed_ozone_tod_in) ) then
       start_tod = prescribed_ozone_tod_in
    endif

    if (len_trim(filename) > 0 ) has_prescribed_ozone = .true.

  endsubroutine prescribed_ozone_setopts

!-------------------------------------------------------------------
!-------------------------------------------------------------------
  subroutine prescribed_ozone_defaultopts(   &
       prescribed_ozone_file_out,     &
       prescribed_ozone_filelist_out, &
       prescribed_ozone_datapath_out, &
       prescribed_ozone_type_out,     &
       prescribed_ozone_name_out,&
       prescribed_ozone_rmfile_out,   &
       prescribed_ozone_ymd_out,      &
       prescribed_ozone_tod_out       &
       ) 

    implicit none

    character(len=*), intent(out), optional :: prescribed_ozone_file_out
    character(len=*), intent(out), optional :: prescribed_ozone_filelist_out
    character(len=*), intent(out), optional :: prescribed_ozone_datapath_out
    character(len=*), intent(out), optional :: prescribed_ozone_type_out
    character(len=*), intent(out), optional :: prescribed_ozone_name_out
    logical,          intent(out), optional :: prescribed_ozone_rmfile_out
    integer,          intent(out), optional :: prescribed_ozone_ymd_out
    integer,          intent(out), optional :: prescribed_ozone_tod_out

    if ( present(prescribed_ozone_file_out) ) then
       prescribed_ozone_file_out = filename
    endif
    if ( present(prescribed_ozone_filelist_out) ) then
       prescribed_ozone_filelist_out = filelist
    endif
    if ( present(prescribed_ozone_datapath_out) ) then
       prescribed_ozone_datapath_out = datapath
    endif
    if ( present(prescribed_ozone_type_out) ) then
       prescribed_ozone_type_out = data_type
    endif
    if ( present(prescribed_ozone_name_out) ) then
       prescribed_ozone_name_out = fld_name
    endif
    if ( present(prescribed_ozone_rmfile_out) ) then
       prescribed_ozone_rmfile_out = rmv_file
    endif
    if ( present(prescribed_ozone_ymd_out) ) then
       prescribed_ozone_ymd_out = start_ymd
    endif
    if ( present(prescribed_ozone_tod_out) ) then
       prescribed_ozone_tod_out = start_tod
    endif

  endsubroutine prescribed_ozone_defaultopts

!-------------------------------------------------------------------
!-------------------------------------------------------------------
  subroutine prescribed_ozone_adv( state )

    use tracer_data,  only : advance_trcdata
    use physics_types,only : physics_state
    use ppgrid,       only : begchunk, endchunk
    use ppgrid,       only : pcols, pver
    use string_utils, only : to_lower, GLC
    use cam_history,  only : outfld
    use physconst,    only : amass => mwdry       ! molecular weight dry air ~ kg/kmole
    use physconst,    only : boltz                ! J/K/molecule

    implicit none

    type(physics_state), intent(in):: state(begchunk:endchunk)                 
    integer :: c,ncol
    real(r8) :: to_mmr(pcols,pver)
    real(r8), parameter :: molmass = 47.9981995_r8

    if( .not. has_prescribed_ozone ) return

    call advance_trcdata( fields, file, state  )

    ! copy prescribed tracer fields into state svariable with the correct units
    do c = begchunk,endchunk
       ncol = state(c)%ncol
       select case ( to_lower(trim(fields(1)%units(:GLC(fields(1)%units)))) )
       case ("molec/cm3","/cm3","molecules/cm3","cm^-3","cm**-3")
          to_mmr(:ncol,:) = (molmass*1.e6_r8*boltz*state(c)%t(:ncol,:))/(amass*state(c)%pmiddry(:ncol,:))
       case ('kg/kg','mmr')
          to_mmr(:ncol,:) = 1._r8
       case ('mol/mol','mole/mole','vmr','fraction')
          to_mmr(:ncol,:) = molmass/amass
       case default
          write(iulog,*) 'prescribed_ozone_adv: units = ',trim(fields(1)%units) ,' are not recognized'
          call endrun('prescribed_ozone_adv: units are not recognized')
       end select

       fields(1)%data(:ncol,:,c+fields(1)%chnk_offset) = to_mmr(:ncol,:) * fields(1)%data(:ncol,:,c+fields(1)%chnk_offset)      ! mmr

       call outfld( ozone_name, (amass/molmass)*fields(1)%data(:ncol,:,c+fields(1)%chnk_offset), ncol, state(c)%lchnk )

    enddo

  end subroutine prescribed_ozone_adv

!-------------------------------------------------------------------

  subroutine init_prescribed_ozone_restart( piofile )
    use pio, only : file_desc_t
    use tracer_data, only : init_trc_restart
    implicit none
    type(file_desc_t),intent(inout) :: pioFile     ! pio File pointer

    call init_trc_restart( 'prescribed_ozone', piofile, file )

  end subroutine init_prescribed_ozone_restart
!-------------------------------------------------------------------
  subroutine write_prescribed_ozone_restart( piofile )
    use tracer_data, only : write_trc_restart
    use pio, only : file_desc_t
    implicit none

    type(file_desc_t) :: piofile

    call write_trc_restart( piofile, file )

  end subroutine write_prescribed_ozone_restart

!-------------------------------------------------------------------
  subroutine read_prescribed_ozone_restart( pioFile )
    use tracer_data, only : read_trc_restart
    use pio, only : file_desc_t
    implicit none

    type(file_desc_t) :: piofile
    
    call read_trc_restart( 'prescribed_ozone', piofile, file )

  end subroutine read_prescribed_ozone_restart

end module prescribed_ozone
